#!/usr/bin/env python
# encoding: utf-8

"""
PYTHON DLA INŻYNIERÓW
Lekcja: #4 Krotki (Tuples)

Copyright: www.oprojektowaniu.pl | 2018

Spis treści:
1. Tworzenie krotek
2. Podstawowe operacje i metody na krotkach
3. Niezmienność krotek
4. Kiedy stosować krotki?
5. Przykład zastosowania krotek
"""


# ==========================================================================
# 1. Tworzenie krotek (Tuples)
# ==========================================================================

# Tworzenie krotki (tuple)
tup = ('jeden', 'dwa', 'trzy')
print(tup)

# Krotki możesz tworzyć również w ten sposób:
tup2 = 'cztery', 'pięć', 'sześć'    # to też jest krotka
print(tup2)

# Krotka stworzona z różnych typów
tup_mixed = (1, 'dwa', [3, 4.1])
print(tup_mixed)

# ==========================================================================
# 2. Podstawowe operacje i metody na krotkach
# ==========================================================================

# --------------------------------------------------------------------------
# Indeksowanie i "wycinanie" elementów
# --------------------------------------------------------------------------

print(tup[0])    # odwołanie do pierwszego elementu krotki
print(tup[-1])   # odwołanie do ostatniego elementu
print(tup[1:])   # wyodrębnienie elementów krotki od drugiego do ostatniego</pre>

# tup[1] = 'zmień element'

# --------------------------------------------------------------------------
# Łączenie i powielanie krotek
# --------------------------------------------------------------------------

tup1 = (1,2,3)
print(tup1)
typ2 = ('a', 'b', 'c')
print(tup2)

# Łączenie krotek
print(tup1 + tup2)

# lub:
tup_new = tup2 + tup1   # tutaj w odwrotnej kolejności
print(tup_new)

# Powielanie krotek
print(('a', 'b', 'c')*3)


# --------------------------------------------------------------------------
# Metody wbudowane
# --------------------------------------------------------------------------

tup = ('jeden', 'dwa', 'trzy')
print(tup)

# Zwraca liczbę elementów krotki
print(len(tup))

# Zwraca indeks pierwszego wystąpienia szukanego elementu w krotce
print(tup.index('dwa'))

# Zwraca liczbę wystąpień elementu w krotce
print(tup.count('dwa'))

# Prawda (True) jeśli element występuje, w przeciwnym przypadku fałsz (False)
print('element' in tup)

# Zwraca najmniejszy min() i największy max() element krotki
tup_numbers = (1,2,3)   # żeby użyć tych metod krotka musi zawierać tylko elementy liczbowe
print(tup_numbers)
print(min(tup_numbers))
print(max(tup_numbers))

# Zwraca sumę elementów krotki
print(sum(tup_numbers))


# ==========================================================================
# 3. Niezmienność krotek
# ==========================================================================

# Dodawanie elementu do krotki
# tup.append('dodaj element')

# Usuwanie elementu krotki
# tup.remove('jeden')


# ==========================================================================
# 4. Kiedy stosować krotki?
# ==========================================================================

'''
Muszę tutaj przyznać, że krotek nie używa się zbyt często. Są one wykorzystywane w programowaniu rzadziej niż listy i do tego w specyficznych zastosowaniach. A oto kilka zalet krotek.

- Po pierwsze, krotki są znacznie szybsze niż listy. Różnica jest nawet 10-krotna (krótki test znajdziesz [tutaj](https://stackoverflow.com/questions/68630/are-tuples-more-efficient-than-lists-in-python)).
- Krotki używamy kiedy mamy do czynienia z elementami, które nie powinny być podatne na modyfikacje, jak np. dni tygodnia lub data w kalendarzu. Krotki zapewniają bezpieczeństwo, że żaden z jej elementów jak i ona sama nie zmienią się podczas przeprowadzanych operacji. Listy tego nie zapewniają.
- Zasada generalne jest też taka, że krotki używamy do różnych typów danych (*heterogeneous datatypes*) a listy do podobnych typów danych (*homogeneous datatypes*).
- Dodatkowo, krotki możemy używać w tzw. słownikach jako ich klucze (o tym będzie w następnej lekcji).
'''


# ==========================================================================
# 5. Przykład zastosowania krotek
# ==========================================================================

# 'Zbiorowe' definiowanie zmiennych
belka1 = ('C30/37', 0.3, 0.5)	# parametry belki: klasa betonu, szerokość, wysokość
(klasa_bet, b, h) = belka1

# Teraz można bezpośrednio odwoływać się do poszczególnych parametrów belki
print(klasa_bet)
print(b)
print(h)